@@ -269,6 +269,8 @@ class transfer_op
                 std::move(*this));
     }
 
+    static bool never_pending_;
+
 public:
     template<class Handler_>
     transfer_op(
@@ -278,10 +280,26 @@ public:
         : async_base<Handler, Executor>(
             std::forward<Handler_>(h), s.get_executor())
         , impl_(s.impl_)
-        , pg_(state().pending)
+        , pg_()
         , b_(b)
     {
-        (*this)({});
+        this->set_allowed_cancellation(net::cancellation_type::all);
+        if (buffer_bytes(b_) == 0 && state().pending)
+        {
+            // Workaround:
+            // Corner case discovered in https://github.com/boostorg/beast/issues/2065
+            // Enclosing SSL stream wishes to complete a 0-length write early by
+            // executing a 0-length read against the underlying stream.
+            // This can occur even if an existing async_read is in progress.
+            // In this specific case, we will complete the async op with no error
+            // in order to prevent assertions and/or internal corruption of the basic_stream
+            this->complete(false, error_code(), 0);
+        }
+        else
+        {
+            pg_.assign(state().pending);
+            (*this)({});
+        }
     }
 
     void
@@ -296,25 +314,39 @@ public:
             {
                 // make sure we perform the no-op
                 BOOST_ASIO_CORO_YIELD
-                async_perform(0, is_read{});
+                {
+                    BOOST_ASIO_HANDLER_LOCATION((
+                        __FILE__, __LINE__,
+                        (isRead ? "basic_stream::async_read_some"
+                            : "basic_stream::async_write_some")));
+
+                    async_perform(0, is_read{});
+                }
                 // apply the timeout manually, otherwise
                 // behavior varies across platforms.
                 if(state().timer.expiry() <= clock_type::now())
                 {
                     impl_->close();
-                    ec = beast::error::timeout;
+                    BOOST_BEAST_ASSIGN_EC(ec, beast::error::timeout);
                 }
                 goto upcall;
             }
 
             // if a timeout is active, wait on the timer
             if(state().timer.expiry() != never())
+            {
+                BOOST_ASIO_HANDLER_LOCATION((
+                    __FILE__, __LINE__,
+                    (isRead ? "basic_stream::async_read_some"
+                        : "basic_stream::async_write_some")));
+
                 state().timer.async_wait(
                     timeout_handler<decltype(this->get_executor())>{
                         state(),
                         impl_,
                         state().tick,
                         this->get_executor()});
+            }
 
             // check rate limit, maybe wait
             std::size_t amount;
@@ -323,7 +355,14 @@ public:
             {
                 ++impl_->waiting;
                 BOOST_ASIO_CORO_YIELD
-                impl_->timer.async_wait(std::move(*this));
+                {
+                    BOOST_ASIO_HANDLER_LOCATION((
+                        __FILE__, __LINE__,
+                        (isRead ? "basic_stream::async_read_some"
+                            : "basic_stream::async_write_some")));
+
+                    impl_->timer.async_wait(std::move(*this));
+                }
                 if(ec)
                 {
                     // socket was closed, or a timeout
@@ -333,7 +372,7 @@ public:
                     if(state().timeout)
                     {
                         // yes, socket already closed
-                        ec = beast::error::timeout;
+                        BOOST_BEAST_ASSIGN_EC(ec, beast::error::timeout);
                         state().timeout = false;
                     }
                     goto upcall;
@@ -347,7 +386,14 @@ public:
             }
 
             BOOST_ASIO_CORO_YIELD
-            async_perform(amount, is_read{});
+            {
+                BOOST_ASIO_HANDLER_LOCATION((
+                    __FILE__, __LINE__,
+                    (isRead ? "basic_stream::async_read_some"
+                        : "basic_stream::async_write_some")));
+
+                async_perform(amount, is_read{});
+            }
 
             if(state().timer.expiry() != never())
             {
@@ -362,7 +408,7 @@ public:
                     if(state().timeout)
                     {
                         // yes, socket already closed
-                        ec = beast::error::timeout;
+                        BOOST_BEAST_ASSIGN_EC(ec, beast::error::timeout);
                         state().timeout = false;
                     }
                 }
@@ -407,13 +453,24 @@ public:
         , pg0_(impl_->read.pending)
         , pg1_(impl_->write.pending)
     {
+        this->set_allowed_cancellation(net::cancellation_type::all);
         if(state().timer.expiry() != stream_base::never())
+        {
+            BOOST_ASIO_HANDLER_LOCATION((
+                __FILE__, __LINE__,
+                "basic_stream::async_connect"));
+
             impl_->write.timer.async_wait(
                 timeout_handler<decltype(this->get_executor())>{
                     state(),
                     impl_,
                     state().tick,
                     this->get_executor()});
+        }
+
+        BOOST_ASIO_HANDLER_LOCATION((
+            __FILE__, __LINE__,
+            "basic_stream::async_connect"));
 
         impl_->socket.async_connect(
             ep, std::move(*this));
@@ -434,13 +491,24 @@ public:
         , pg0_(impl_->read.pending)
         , pg1_(impl_->write.pending)
     {
+        this->set_allowed_cancellation(net::cancellation_type::all);
         if(state().timer.expiry() != stream_base::never())
+        {
+            BOOST_ASIO_HANDLER_LOCATION((
+                __FILE__, __LINE__,
+                "basic_stream::async_connect"));
+
             impl_->write.timer.async_wait(
                 timeout_handler<decltype(this->get_executor())>{
                     state(),
                     impl_,
                     state().tick,
                     this->get_executor()});
+        }
+
+        BOOST_ASIO_HANDLER_LOCATION((
+            __FILE__, __LINE__,
+            "basic_stream::async_connect"));
 
         net::async_connect(impl_->socket,
             eps, cond, std::move(*this));
@@ -461,13 +529,24 @@ public:
         , pg0_(impl_->read.pending)
         , pg1_(impl_->write.pending)
     {
+        this->set_allowed_cancellation(net::cancellation_type::all);
         if(state().timer.expiry() != stream_base::never())
+        {
+            BOOST_ASIO_HANDLER_LOCATION((
+                __FILE__, __LINE__,
+                "basic_stream::async_connect"));
+
             impl_->write.timer.async_wait(
                 timeout_handler<decltype(this->get_executor())>{
                     state(),
                     impl_,
                     state().tick,
                     this->get_executor()});
+        }
+
+        BOOST_ASIO_HANDLER_LOCATION((
+            __FILE__, __LINE__,
+            "basic_stream::async_connect"));
 
         net::async_connect(impl_->socket,
             begin, end, cond, std::move(*this));
@@ -491,7 +570,7 @@ public:
                 if(state().timeout)
                 {
                     // yes, socket already closed
-                    ec = beast::error::timeout;
+                    BOOST_BEAST_ASSIGN_EC(ec, beast::error::timeout);
                     state().timeout = false;
                 }
             }
@@ -689,6 +768,14 @@ basic_stream(basic_stream&& other)
     // controlling its lifetime.
 }
 
+template<class Protocol, class Executor, class RatePolicy>
+template<class Executor_>
+basic_stream<Protocol, Executor, RatePolicy>::
+basic_stream(basic_stream<Protocol, Executor_, RatePolicy> && other)
+    : impl_(boost::make_shared<impl_type>(std::false_type{}, std::move(other.impl_->socket)))
+{
+}
+
 //------------------------------------------------------------------------------
 
 template<class Protocol, class Executor, class RatePolicy>
