@@ -12,16 +12,21 @@
 // This file has no include guards or namespaces - it's expanded inline inside default_ops.hpp
 //
 
+#include <boost/multiprecision/detail/standalone_config.hpp>
+#include <boost/multiprecision/detail/no_exceptions_support.hpp>
+#include <boost/multiprecision/detail/assert.hpp>
+
 #ifdef BOOST_MSVC
 #pragma warning(push)
 #pragma warning(disable : 6326) // comparison of two constants
+#pragma warning(disable : 4127) // conditional expression is constant
 #endif
 
 template <class T>
 void hyp0F1(T& result, const T& b, const T& x)
 {
-   typedef typename boost::multiprecision::detail::canonical<boost::int32_t, T>::type  si_type;
-   typedef typename boost::multiprecision::detail::canonical<boost::uint32_t, T>::type ui_type;
+   using si_type = typename boost::multiprecision::detail::canonical<std::int32_t, T>::type ;
+   using ui_type = typename boost::multiprecision::detail::canonical<std::uint32_t, T>::type;
 
    // Compute the series representation of Hypergeometric0F1 taken from
    // http://functions.wolfram.com/HypergeometricFunctions/Hypergeometric0F1/06/01/01/
@@ -67,13 +72,94 @@ void hyp0F1(T& result, const T& b, const T& x)
    }
 
    if (n >= series_limit)
-      BOOST_THROW_EXCEPTION(std::runtime_error("H0F1 Failed to Converge"));
+      BOOST_MP_THROW_EXCEPTION(std::runtime_error("H0F1 Failed to Converge"));
+}
+
+template <class T, unsigned N, bool b = boost::multiprecision::detail::is_variable_precision<boost::multiprecision::number<T> >::value>
+struct scoped_N_precision
+{
+   template <class U>
+   scoped_N_precision(U const&) {}
+   template <class U>
+   void reduce(U&) {}
+};
+
+template <class T, unsigned N>
+struct scoped_N_precision<T, N, true>
+{
+   unsigned old_precision, old_arg_precision;
+   scoped_N_precision(T& arg)
+   {
+      old_precision     = T::thread_default_precision();
+      old_arg_precision = arg.precision();
+      T::thread_default_precision(old_arg_precision * N);
+      arg.precision(old_arg_precision * N);
+   }
+   ~scoped_N_precision()
+   {
+      T::thread_default_precision(old_precision);
+   }
+   void reduce(T& arg) 
+   {
+      arg.precision(old_arg_precision);
+   }
+};
+
+template <class T>
+void reduce_n_half_pi(T& arg, const T& n, bool go_down)
+{
+   //
+   // We need to perform argument reduction at 3 times the precision of arg
+   // in order to ensure a correct result up to arg = 1/epsilon.  Beyond that
+   // the value of n will have been incorrectly calculated anyway since it will
+   // have a value greater than 1/epsilon and no longer be an exact integer value.
+   //
+   // More information in ARGUMENT REDUCTION FOR HUGE ARGUMENTS. K C Ng.
+   //
+   // There are two mutually exclusive ways to achieve this, both of which are 
+   // supported here:
+   // 1) To define a fixed precision type with 3 times the precision for the calculation.
+   // 2) To dynamically increase the precision of the variables.
+   //
+   using reduction_type = typename boost::multiprecision::detail::transcendental_reduction_type<T>::type;
+   //
+   // Make a copy of the arg at higher precision:
+   //
+   reduction_type big_arg(arg);
+   //
+   // Dynamically increase precision when supported, this increases the default
+   // and ups the precision of big_arg to match:
+   //
+   scoped_N_precision<T, 3> scoped_precision(big_arg);
+   //
+   // High precision PI:
+   //
+   reduction_type reduction = get_constant_pi<reduction_type>();
+   eval_ldexp(reduction, reduction, -1); // divide by 2
+   eval_multiply(reduction, n);
+
+   BOOST_MATH_INSTRUMENT_CODE(big_arg.str(10, std::ios_base::scientific));
+   BOOST_MATH_INSTRUMENT_CODE(reduction.str(10, std::ios_base::scientific));
+
+   if (go_down)
+      eval_subtract(big_arg, reduction, big_arg);
+   else
+      eval_subtract(big_arg, reduction);
+   arg = T(big_arg);
+   //
+   // If arg is a variable precision type, then we have just copied the
+   // precision of big_arg s well it's value.  Reduce the precision now:
+   //
+   scoped_precision.reduce(arg);
+   BOOST_MATH_INSTRUMENT_CODE(big_arg.str(10, std::ios_base::scientific));
+   BOOST_MATH_INSTRUMENT_CODE(arg.str(10, std::ios_base::scientific));
 }
 
 template <class T>
 void eval_sin(T& result, const T& x)
 {
-   BOOST_STATIC_ASSERT_MSG(number_category<T>::value == number_kind_floating_point, "The sin function is only valid for floating point types.");
+   static_assert(number_category<T>::value == number_kind_floating_point, "The sin function is only valid for floating point types.");
+   BOOST_MATH_INSTRUMENT_CODE(x.str(0, std::ios_base::scientific));
    if (&result == &x)
    {
       T temp;
@@ -82,21 +168,21 @@ void eval_sin(T& result, const T& x)
       return;
    }
 
-   typedef typename boost::multiprecision::detail::canonical<boost::int32_t, T>::type  si_type;
-   typedef typename boost::multiprecision::detail::canonical<boost::uint32_t, T>::type ui_type;
-   typedef typename mpl::front<typename T::float_types>::type                          fp_type;
+   using si_type = typename boost::multiprecision::detail::canonical<std::int32_t, T>::type ;
+   using ui_type = typename boost::multiprecision::detail::canonical<std::uint32_t, T>::type;
+   using fp_type = typename std::tuple_element<0, typename T::float_types>::type                         ;
 
    switch (eval_fpclassify(x))
    {
    case FP_INFINITE:
    case FP_NAN:
-      if (std::numeric_limits<number<T, et_on> >::has_quiet_NaN)
+      BOOST_IF_CONSTEXPR(std::numeric_limits<number<T, et_on> >::has_quiet_NaN)
       {
          result = std::numeric_limits<number<T, et_on> >::quiet_NaN().backend();
          errno  = EDOM;
       }
       else
-         BOOST_THROW_EXCEPTION(std::domain_error("Result is undefined or complex and there is no NaN for this number type."));
+         BOOST_MP_THROW_EXCEPTION(std::domain_error("Result is undefined or complex and there is no NaN for this number type."));
       return;
    case FP_ZERO:
       result = x;
@@ -119,49 +205,84 @@ void eval_sin(T& result, const T& x)
    }
 
    T n_pi, t;
-   // Remove even multiples of pi.
-   if (xx.compare(get_constant_pi<T>()) > 0)
+   T half_pi = get_constant_pi<T>();
+   eval_ldexp(half_pi, half_pi, -1); // divide by 2
+   // Remove multiples of pi/2.
+   if (xx.compare(half_pi) > 0)
    {
-      eval_divide(n_pi, xx, get_constant_pi<T>());
+      eval_divide(n_pi, xx, half_pi);
       eval_trunc(n_pi, n_pi);
-      t = ui_type(2);
+      t = ui_type(4);
       eval_fmod(t, n_pi, t);
-      const bool b_n_pi_is_even = eval_get_sign(t) == 0;
-      eval_multiply(n_pi, get_constant_pi<T>());
+      bool b_go_down = false;
+      if (t.compare(ui_type(1)) == 0)
+      {
+         b_go_down = true;
+      }
+      else if (t.compare(ui_type(2)) == 0)
+      {
+         b_negate_sin = !b_negate_sin;
+      }
+      else if (t.compare(ui_type(3)) == 0)
+      {
+         b_negate_sin = !b_negate_sin;
+         b_go_down    = true;
+      }
+
+      if (b_go_down)
+         eval_increment(n_pi);
+      //
+      // If n_pi is > 1/epsilon, then it is no longer an exact integer value
+      // but an approximation.  As a result we can no longer reliably reduce
+      // xx to 0 <= xx < pi/2, nor can we tell the sign of the result as we need
+      // n_pi % 4 for that, but that will always be zero in this situation.
+      // We could use a higher precision type for n_pi, along with division at
+      // higher precision, but that's rather expensive.  So for now we do not support
+      // this, and will see if anyone complains and has a legitimate use case.
+      //
       if (n_pi.compare(get_constant_one_over_epsilon<T>()) > 0)
       {
          result = ui_type(0);
          return;
       }
-      else
-         eval_subtract(xx, n_pi);
+
+      reduce_n_half_pi(xx, n_pi, b_go_down);
+      //
+      // Post reduction we may be a few ulp below zero or above pi/2
+      // given that n_pi was calculated at working precision and not
+      // at the higher precision used for reduction.  Correct that now:
+      //
+      if (eval_get_sign(xx) < 0)
+      {
+         xx.negate();
+         b_negate_sin = !b_negate_sin;
+      }
+      if (xx.compare(half_pi) > 0)
+      {
+         eval_ldexp(half_pi, half_pi, 1);
+         eval_subtract(xx, half_pi, xx);
+         eval_ldexp(half_pi, half_pi, -1);
+         b_go_down = !b_go_down;
+      }
 
       BOOST_MATH_INSTRUMENT_CODE(xx.str(0, std::ios_base::scientific));
       BOOST_MATH_INSTRUMENT_CODE(n_pi.str(0, std::ios_base::scientific));
-
-      // Adjust signs if the multiple of pi is not even.
-      if (!b_n_pi_is_even)
-      {
-         b_negate_sin = !b_negate_sin;
-      }
-   }
-
-   // Reduce the argument to 0 <= xx <= pi/2.
-   eval_ldexp(t, get_constant_pi<T>(), -1);
-   if (xx.compare(t) > 0)
-   {
-      eval_subtract(xx, get_constant_pi<T>(), xx);
-      BOOST_MATH_INSTRUMENT_CODE(xx.str(0, std::ios_base::scientific));
+      BOOST_MP_ASSERT(xx.compare(half_pi) <= 0);
+      BOOST_MP_ASSERT(xx.compare(ui_type(0)) >= 0);
    }
 
+   t = half_pi;
    eval_subtract(t, xx);
+
    const bool b_zero    = eval_get_sign(xx) == 0;
    const bool b_pi_half = eval_get_sign(t) == 0;
 
+   BOOST_MATH_INSTRUMENT_CODE(xx.str(0, std::ios_base::scientific));
+   BOOST_MATH_INSTRUMENT_CODE(t.str(0, std::ios_base::scientific));
+
    // Check if the reduced argument is very close to 0 or pi/2.
    const bool b_near_zero    = xx.compare(fp_type(1e-1)) < 0;
    const bool b_near_pi_half = t.compare(fp_type(1e-1)) < 0;
-   ;
 
    if (b_zero)
    {
@@ -197,8 +318,8 @@ void eval_sin(T& result, const T& x)
       // divide by three identity a certain number of times.
       // Here we use division by 3^9 --> (19683 = 3^9).
 
-      static const si_type n_scale           = 9;
-      static const si_type n_three_pow_scale = static_cast<si_type>(19683L);
+      constexpr si_type n_scale           = 9;
+      constexpr si_type n_three_pow_scale = static_cast<si_type>(19683L);
 
       eval_divide(xx, n_three_pow_scale);
 
@@ -212,7 +333,7 @@ void eval_sin(T& result, const T& x)
       eval_multiply(result, xx);
 
       // Convert back using multiple angle identity.
-      for (boost::int32_t k = static_cast<boost::int32_t>(0); k < n_scale; k++)
+      for (std::int32_t k = static_cast<std::int32_t>(0); k < n_scale; k++)
       {
          // Rescale the cosine value using the multiple angle identity.
          eval_multiply(t2, result, ui_type(3));
@@ -225,12 +346,13 @@ void eval_sin(T& result, const T& x)
 
    if (b_negate_sin)
       result.negate();
+   BOOST_MATH_INSTRUMENT_CODE(result.str(0, std::ios_base::scientific));
 }
 
 template <class T>
 void eval_cos(T& result, const T& x)
 {
-   BOOST_STATIC_ASSERT_MSG(number_category<T>::value == number_kind_floating_point, "The cos function is only valid for floating point types.");
+   static_assert(number_category<T>::value == number_kind_floating_point, "The cos function is only valid for floating point types.");
    if (&result == &x)
    {
       T temp;
@@ -239,21 +361,20 @@ void eval_cos(T& result, const T& x)
       return;
    }
 
-   typedef typename boost::multiprecision::detail::canonical<boost::int32_t, T>::type  si_type;
-   typedef typename boost::multiprecision::detail::canonical<boost::uint32_t, T>::type ui_type;
-   typedef typename mpl::front<typename T::float_types>::type                          fp_type;
+   using si_type = typename boost::multiprecision::detail::canonical<std::int32_t, T>::type ;
+   using ui_type = typename boost::multiprecision::detail::canonical<std::uint32_t, T>::type;
 
    switch (eval_fpclassify(x))
    {
    case FP_INFINITE:
    case FP_NAN:
-      if (std::numeric_limits<number<T, et_on> >::has_quiet_NaN)
+      BOOST_IF_CONSTEXPR(std::numeric_limits<number<T, et_on> >::has_quiet_NaN)
       {
          result = std::numeric_limits<number<T, et_on> >::quiet_NaN().backend();
          errno  = EDOM;
       }
       else
-         BOOST_THROW_EXCEPTION(std::domain_error("Result is undefined or complex and there is no NaN for this number type."));
+         BOOST_MP_THROW_EXCEPTION(std::domain_error("Result is undefined or complex and there is no NaN for this number type."));
       return;
    case FP_ZERO:
       result = ui_type(1);
@@ -273,88 +394,101 @@ void eval_cos(T& result, const T& x)
    {
       xx.negate();
    }
+   BOOST_MATH_INSTRUMENT_CODE(xx.str(0, std::ios_base::scientific));
 
    T n_pi, t;
+   T half_pi = get_constant_pi<T>();
+   eval_ldexp(half_pi, half_pi, -1); // divide by 2
    // Remove even multiples of pi.
-   if (xx.compare(get_constant_pi<T>()) > 0)
+   if (xx.compare(half_pi) > 0)
    {
-      eval_divide(t, xx, get_constant_pi<T>());
+      eval_divide(t, xx, half_pi);
       eval_trunc(n_pi, t);
-      BOOST_MATH_INSTRUMENT_CODE(n_pi.str(0, std::ios_base::scientific));
-      eval_multiply(t, n_pi, get_constant_pi<T>());
-      BOOST_MATH_INSTRUMENT_CODE(t.str(0, std::ios_base::scientific));
       //
-      // If t is so large that all digits cancel the result of this subtraction
-      // is completely meaningless, just assume the result is zero for now...
-      //
-      // TODO We should of course do much better, see:
-      // "ARGUMENT REDUCTION FOR HUGE ARGUMENTS" K C Ng 1992
+      // If n_pi is > 1/epsilon, then it is no longer an exact integer value
+      // but an approximation.  As a result we can no longer reliably reduce
+      // xx to 0 <= xx < pi/2, nor can we tell the sign of the result as we need
+      // n_pi % 4 for that, but that will always be zero in this situation.
+      // We could use a higher precision type for n_pi, along with division at
+      // higher precision, but that's rather expensive.  So for now we do not support
+      // this, and will see if anyone complains and has a legitimate use case.
       //
       if (n_pi.compare(get_constant_one_over_epsilon<T>()) > 0)
       {
          result = ui_type(1);
          return;
       }
-      else
-         eval_subtract(xx, t);
-      BOOST_MATH_INSTRUMENT_CODE(xx.str(0, std::ios_base::scientific));
-
-      // Adjust signs if the multiple of pi is not even.
-      t = ui_type(2);
+      BOOST_MATH_INSTRUMENT_CODE(n_pi.str(0, std::ios_base::scientific));
+      t = ui_type(4);
       eval_fmod(t, n_pi, t);
-      const bool b_n_pi_is_even = eval_get_sign(t) == 0;
 
-      if (!b_n_pi_is_even)
+      bool b_go_down = false;
+      if (t.compare(ui_type(0)) == 0)
       {
+         b_go_down = true;
+      }
+      else if (t.compare(ui_type(1)) == 0)
+      {
+         b_negate_cos = true;
+      }
+      else if (t.compare(ui_type(2)) == 0)
+      {
+         b_go_down    = true;
+         b_negate_cos = true;
+      }
+      else
+      {
+         BOOST_MP_ASSERT(t.compare(ui_type(3)) == 0);
+      }
+
+      if (b_go_down)
+         eval_increment(n_pi);
+
+      reduce_n_half_pi(xx, n_pi, b_go_down);
+      //
+      // Post reduction we may be a few ulp below zero or above pi/2
+      // given that n_pi was calculated at working precision and not
+      // at the higher precision used for reduction.  Correct that now:
+      //
+      if (eval_get_sign(xx) < 0)
+      {
+         xx.negate();
          b_negate_cos = !b_negate_cos;
       }
-   }
-
-   // Reduce the argument to 0 <= xx <= pi/2.
-   eval_ldexp(t, get_constant_pi<T>(), -1);
-   int com = xx.compare(t);
-   if (com > 0)
-   {
-      eval_subtract(xx, get_constant_pi<T>(), xx);
-      b_negate_cos = !b_negate_cos;
-      BOOST_MATH_INSTRUMENT_CODE(xx.str(0, std::ios_base::scientific));
-   }
-
-   const bool b_zero    = eval_get_sign(xx) == 0;
-   const bool b_pi_half = com == 0;
-
-   // Check if the reduced argument is very close to 0.
-   const bool b_near_zero = xx.compare(fp_type(1e-1)) < 0;
-
-   if (b_zero)
-   {
-      result = si_type(1);
-   }
-   else if (b_pi_half)
-   {
-      result = si_type(0);
-   }
-   else if (b_near_zero)
-   {
-      eval_multiply(t, xx, xx);
-      eval_divide(t, si_type(-4));
-      n_pi = fp_type(0.5f);
-      hyp0F1(result, n_pi, t);
-      BOOST_MATH_INSTRUMENT_CODE(result.str(0, std::ios_base::scientific));
+      if (xx.compare(half_pi) > 0)
+      {
+         eval_ldexp(half_pi, half_pi, 1);
+         eval_subtract(xx, half_pi, xx);
+         eval_ldexp(half_pi, half_pi, -1);
+      }
+      BOOST_MP_ASSERT(xx.compare(half_pi) <= 0);
+      BOOST_MP_ASSERT(xx.compare(ui_type(0)) >= 0);
    }
    else
    {
-      eval_subtract(t, xx);
-      eval_sin(result, t);
+      n_pi = ui_type(1);
+      reduce_n_half_pi(xx, n_pi, true);
+   }
+
+   const bool b_zero = eval_get_sign(xx) == 0;
+
+   if (b_zero)
+   {
+      result = si_type(0);
+   }
+   else
+   {
+      eval_sin(result, xx);
    }
    if (b_negate_cos)
       result.negate();
+   BOOST_MATH_INSTRUMENT_CODE(result.str(0, std::ios_base::scientific));
 }
 
 template <class T>
 void eval_tan(T& result, const T& x)
 {
-   BOOST_STATIC_ASSERT_MSG(number_category<T>::value == number_kind_floating_point, "The tan function is only valid for floating point types.");
+   static_assert(number_category<T>::value == number_kind_floating_point, "The tan function is only valid for floating point types.");
    if (&result == &x)
    {
       T temp;
@@ -375,7 +509,7 @@ void hyp2F1(T& result, const T& a, const T& b, const T& c, const T& x)
    // Abramowitz and Stegun 15.1.1.
    // There are no checks on input range or parameter boundaries.
 
-   typedef typename boost::multiprecision::detail::canonical<boost::uint32_t, T>::type ui_type;
+   using ui_type = typename boost::multiprecision::detail::canonical<std::uint32_t, T>::type;
 
    T x_pow_n_div_n_fact(x);
    T pochham_a(a);
@@ -427,15 +561,15 @@ void hyp2F1(T& result, const T& a, const T& b, const T& c, const T& x)
          break;
    }
    if (n > series_limit)
-      BOOST_THROW_EXCEPTION(std::runtime_error("H2F1 failed to converge."));
+      BOOST_MP_THROW_EXCEPTION(std::runtime_error("H2F1 failed to converge."));
 }
 
 template <class T>
 void eval_asin(T& result, const T& x)
 {
-   BOOST_STATIC_ASSERT_MSG(number_category<T>::value == number_kind_floating_point, "The asin function is only valid for floating point types.");
-   typedef typename boost::multiprecision::detail::canonical<boost::uint32_t, T>::type ui_type;
-   typedef typename mpl::front<typename T::float_types>::type                          fp_type;
+   static_assert(number_category<T>::value == number_kind_floating_point, "The asin function is only valid for floating point types.");
+   using ui_type = typename boost::multiprecision::detail::canonical<std::uint32_t, T>::type;
+   using fp_type = typename std::tuple_element<0, typename T::float_types>::type                         ;
 
    if (&result == &x)
    {
@@ -448,13 +582,13 @@ void eval_asin(T& result, const T& x)
    {
    case FP_NAN:
    case FP_INFINITE:
-      if (std::numeric_limits<number<T, et_on> >::has_quiet_NaN)
+      BOOST_IF_CONSTEXPR(std::numeric_limits<number<T, et_on> >::has_quiet_NaN)
       {
          result = std::numeric_limits<number<T, et_on> >::quiet_NaN().backend();
          errno  = EDOM;
       }
       else
-         BOOST_THROW_EXCEPTION(std::domain_error("Result is undefined or complex and there is no NaN for this number type."));
+         BOOST_MP_THROW_EXCEPTION(std::domain_error("Result is undefined or complex and there is no NaN for this number type."));
       return;
    case FP_ZERO:
       result = x;
@@ -471,13 +605,13 @@ void eval_asin(T& result, const T& x)
    int c = xx.compare(ui_type(1));
    if (c > 0)
    {
-      if (std::numeric_limits<number<T, et_on> >::has_quiet_NaN)
+      BOOST_IF_CONSTEXPR(std::numeric_limits<number<T, et_on> >::has_quiet_NaN)
       {
          result = std::numeric_limits<number<T, et_on> >::quiet_NaN().backend();
          errno  = EDOM;
       }
       else
-         BOOST_THROW_EXCEPTION(std::domain_error("Result is undefined or complex and there is no NaN for this number type."));
+         BOOST_MP_THROW_EXCEPTION(std::domain_error("Result is undefined or complex and there is no NaN for this number type."));
       return;
    }
    else if (c == 0)
@@ -489,7 +623,7 @@ void eval_asin(T& result, const T& x)
       return;
    }
 
-   if (xx.compare(fp_type(1e-4)) < 0)
+   if (xx.compare(fp_type(1e-3)) < 0)
    {
       // http://functions.wolfram.com/ElementaryFunctions/ArcSin/26/01/01/
       eval_multiply(xx, xx);
@@ -500,8 +634,11 @@ void eval_asin(T& result, const T& x)
       eval_multiply(result, x);
       return;
    }
-   else if (xx.compare(fp_type(1 - 1e-4f)) > 0)
+   else if (xx.compare(fp_type(1 - 5e-2f)) > 0)
    {
+      // http://functions.wolfram.com/ElementaryFunctions/ArcSin/26/01/01/
+      // This branch is simlilar in complexity to Newton iterations down to
+      // the above limit.  It is *much* more accurate.
       T dx1;
       T t1, t2;
       eval_subtract(dx1, ui_type(1), xx);
@@ -520,9 +657,9 @@ void eval_asin(T& result, const T& x)
       return;
    }
 #ifndef BOOST_MATH_NO_LONG_DOUBLE_MATH_FUNCTIONS
-   typedef typename boost::multiprecision::detail::canonical<long double, T>::type guess_type;
+   using guess_type = typename boost::multiprecision::detail::canonical<long double, T>::type;
 #else
-   typedef fp_type guess_type;
+   using guess_type = fp_type;
 #endif
    // Get initial estimate using standard math function asin.
    guess_type dd;
@@ -535,8 +672,10 @@ void eval_asin(T& result, const T& x)
    // have at least 2/3 of the digits correct on the assumption that the correction
    // we've just added will finish the job...
 
-   boost::intmax_t current_precision = eval_ilogb(result);
-   boost::intmax_t target_precision  = current_precision - 1 - (std::numeric_limits<number<T> >::digits * 2) / 3;
+   std::intmax_t current_precision = eval_ilogb(result);
+   std::intmax_t target_precision  = std::numeric_limits<number<T> >::is_specialized ? 
+      current_precision - 1 - (std::numeric_limits<number<T> >::digits * 2) / 3
+      : current_precision - 1 - (boost::multiprecision::detail::digits2<number<T> >::value() * 2) / 3;
 
    // Newton-Raphson iteration
    while (current_precision > target_precision)
@@ -558,20 +697,20 @@ void eval_asin(T& result, const T& x)
 template <class T>
 inline void eval_acos(T& result, const T& x)
 {
-   BOOST_STATIC_ASSERT_MSG(number_category<T>::value == number_kind_floating_point, "The acos function is only valid for floating point types.");
-   typedef typename boost::multiprecision::detail::canonical<boost::uint32_t, T>::type ui_type;
+   static_assert(number_category<T>::value == number_kind_floating_point, "The acos function is only valid for floating point types.");
+   using ui_type = typename boost::multiprecision::detail::canonical<std::uint32_t, T>::type;
 
    switch (eval_fpclassify(x))
    {
    case FP_NAN:
    case FP_INFINITE:
-      if (std::numeric_limits<number<T, et_on> >::has_quiet_NaN)
+      BOOST_IF_CONSTEXPR(std::numeric_limits<number<T, et_on> >::has_quiet_NaN)
       {
          result = std::numeric_limits<number<T, et_on> >::quiet_NaN().backend();
          errno  = EDOM;
       }
       else
-         BOOST_THROW_EXCEPTION(std::domain_error("Result is undefined or complex and there is no NaN for this number type."));
+         BOOST_MP_THROW_EXCEPTION(std::domain_error("Result is undefined or complex and there is no NaN for this number type."));
       return;
    case FP_ZERO:
       result = get_constant_pi<T>();
@@ -579,18 +718,19 @@ inline void eval_acos(T& result, const T& x)
       return;
    }
 
-   eval_abs(result, x);
-   int c = result.compare(ui_type(1));
+   T xx;
+   eval_abs(xx, x);
+   int c = xx.compare(ui_type(1));
 
    if (c > 0)
    {
-      if (std::numeric_limits<number<T, et_on> >::has_quiet_NaN)
+      BOOST_IF_CONSTEXPR(std::numeric_limits<number<T, et_on> >::has_quiet_NaN)
       {
          result = std::numeric_limits<number<T, et_on> >::quiet_NaN().backend();
          errno  = EDOM;
       }
       else
-         BOOST_THROW_EXCEPTION(std::domain_error("Result is undefined or complex and there is no NaN for this number type."));
+         BOOST_MP_THROW_EXCEPTION(std::domain_error("Result is undefined or complex and there is no NaN for this number type."));
       return;
    }
    else if (c == 0)
@@ -602,20 +742,91 @@ inline void eval_acos(T& result, const T& x)
       return;
    }
 
-   eval_asin(result, x);
-   T t;
-   eval_ldexp(t, get_constant_pi<T>(), -1);
-   eval_subtract(result, t);
-   result.negate();
+   using fp_type = typename std::tuple_element<0, typename T::float_types>::type;
+
+   if (xx.compare(fp_type(1e-3)) < 0)
+   {
+      // https://functions.wolfram.com/ElementaryFunctions/ArcCos/26/01/01/
+      eval_multiply(xx, xx);
+      T t1, t2;
+      t1 = fp_type(0.5f);
+      t2 = fp_type(1.5f);
+      hyp2F1(result, t1, t1, t2, xx);
+      eval_multiply(result, x);
+      eval_ldexp(t1, get_constant_pi<T>(), -1);
+      result.negate();
+      eval_add(result, t1);
+      return;
+   }
+   if (eval_get_sign(x) < 0)
+   {
+      eval_acos(result, xx);
+      result.negate();
+      eval_add(result, get_constant_pi<T>());
+      return;
+   }
+   else if (xx.compare(fp_type(0.85)) > 0)
+   {
+      // https://functions.wolfram.com/ElementaryFunctions/ArcCos/26/01/01/
+      // This branch is simlilar in complexity to Newton iterations down to
+      // the above limit.  It is *much* more accurate.
+      T dx1;
+      T t1, t2;
+      eval_subtract(dx1, ui_type(1), xx);
+      t1 = fp_type(0.5f);
+      t2 = fp_type(1.5f);
+      eval_ldexp(dx1, dx1, -1);
+      hyp2F1(result, t1, t1, t2, dx1);
+      eval_ldexp(dx1, dx1, 2);
+      eval_sqrt(t1, dx1);
+      eval_multiply(result, t1);
+      return;
+   }
+
+#ifndef BOOST_MATH_NO_LONG_DOUBLE_MATH_FUNCTIONS
+   using guess_type = typename boost::multiprecision::detail::canonical<long double, T>::type;
+#else
+   using guess_type = fp_type;
+#endif
+   // Get initial estimate using standard math function asin.
+   guess_type dd;
+   eval_convert_to(&dd, xx);
+
+   result = (guess_type)(std::acos(dd));
+
+   // Newton-Raphson iteration, we should double our precision with each iteration,
+   // in practice this seems to not quite work in all cases... so terminate when we
+   // have at least 2/3 of the digits correct on the assumption that the correction
+   // we've just added will finish the job...
+
+   std::intmax_t current_precision = eval_ilogb(result);
+   std::intmax_t target_precision = std::numeric_limits<number<T> >::is_specialized ?
+      current_precision - 1 - (std::numeric_limits<number<T> >::digits * 2) / 3
+      : current_precision - 1 - (boost::multiprecision::detail::digits2<number<T> >::value() * 2) / 3;
+
+   // Newton-Raphson iteration
+   while (current_precision > target_precision)
+   {
+      T sine, cosine;
+      eval_sin(sine, result);
+      eval_cos(cosine, result);
+      eval_subtract(cosine, xx);
+      cosine.negate();
+      eval_divide(cosine, sine);
+      eval_subtract(result, cosine);
+      current_precision = eval_ilogb(cosine);
+      if (current_precision <= (std::numeric_limits<typename T::exponent_type>::min)() + 1)
+         break;
+   }
 }
 
 template <class T>
 void eval_atan(T& result, const T& x)
 {
-   BOOST_STATIC_ASSERT_MSG(number_category<T>::value == number_kind_floating_point, "The atan function is only valid for floating point types.");
-   typedef typename boost::multiprecision::detail::canonical<boost::int32_t, T>::type  si_type;
-   typedef typename boost::multiprecision::detail::canonical<boost::uint32_t, T>::type ui_type;
-   typedef typename mpl::front<typename T::float_types>::type                          fp_type;
+   static_assert(number_category<T>::value == number_kind_floating_point, "The atan function is only valid for floating point types.");
+   using si_type = typename boost::multiprecision::detail::canonical<std::int32_t, T>::type ;
+   using ui_type = typename boost::multiprecision::detail::canonical<std::uint32_t, T>::type;
+   using fp_type = typename std::tuple_element<0, typename T::float_types>::type                         ;
 
    switch (eval_fpclassify(x))
    {
@@ -686,8 +897,10 @@ void eval_atan(T& result, const T& x)
    // have at least 2/3 of the digits correct on the assumption that the correction
    // we've just added will finish the job...
 
-   boost::intmax_t current_precision = eval_ilogb(result);
-   boost::intmax_t target_precision  = current_precision - 1 - (std::numeric_limits<number<T> >::digits * 2) / 3;
+   std::intmax_t current_precision = eval_ilogb(result);
+   std::intmax_t target_precision  = std::numeric_limits<number<T> >::is_specialized ?
+      current_precision - 1 - (std::numeric_limits<number<T> >::digits * 2) / 3
+      : current_precision - 1 - (boost::multiprecision::detail::digits2<number<T> >::value() * 2) / 3;
 
    T s, c, t;
    while (current_precision > target_precision)
@@ -709,7 +922,7 @@ void eval_atan(T& result, const T& x)
 template <class T>
 void eval_atan2(T& result, const T& y, const T& x)
 {
-   BOOST_STATIC_ASSERT_MSG(number_category<T>::value == number_kind_floating_point, "The atan2 function is only valid for floating point types.");
+   static_assert(number_category<T>::value == number_kind_floating_point, "The atan2 function is only valid for floating point types.");
    if (&result == &y)
    {
       T temp(y);
@@ -723,7 +936,7 @@ void eval_atan2(T& result, const T& y, const T& x)
       return;
    }
 
-   typedef typename boost::multiprecision::detail::canonical<boost::uint32_t, T>::type ui_type;
+   using ui_type = typename boost::multiprecision::detail::canonical<std::uint32_t, T>::type;
 
    switch (eval_fpclassify(y))
    {
@@ -821,20 +1034,20 @@ void eval_atan2(T& result, const T& y, const T& x)
    }
 }
 template <class T, class A>
-inline typename enable_if<is_arithmetic<A>, void>::type eval_atan2(T& result, const T& x, const A& a)
+inline typename std::enable_if<boost::multiprecision::detail::is_arithmetic<A>::value, void>::type eval_atan2(T& result, const T& x, const A& a)
 {
-   typedef typename boost::multiprecision::detail::canonical<A, T>::type          canonical_type;
-   typedef typename mpl::if_<is_same<A, canonical_type>, T, canonical_type>::type cast_type;
+   using canonical_type = typename boost::multiprecision::detail::canonical<A, T>::type         ;
+   using cast_type = typename std::conditional<std::is_same<A, canonical_type>::value, T, canonical_type>::type;
    cast_type                                                                      c;
    c = a;
    eval_atan2(result, x, c);
 }
 
 template <class T, class A>
-inline typename enable_if<is_arithmetic<A>, void>::type eval_atan2(T& result, const A& x, const T& a)
+inline typename std::enable_if<boost::multiprecision::detail::is_arithmetic<A>::value, void>::type eval_atan2(T& result, const A& x, const T& a)
 {
-   typedef typename boost::multiprecision::detail::canonical<A, T>::type          canonical_type;
-   typedef typename mpl::if_<is_same<A, canonical_type>, T, canonical_type>::type cast_type;
+   using canonical_type = typename boost::multiprecision::detail::canonical<A, T>::type         ;
+   using cast_type = typename std::conditional<std::is_same<A, canonical_type>::value, T, canonical_type>::type;
    cast_type                                                                      c;
    c = x;
    eval_atan2(result, c, a);
